import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter_typeahead/flutter_typeahead.dart';
import 'package:get/get.dart';
import 'package:sk_base_mobile/app_theme.dart';
import 'package:sk_base_mobile/constants/constants.dart';
import 'package:sk_base_mobile/models/index.dart';
import 'package:sk_base_mobile/screens/new_inventory_inout/components/date_time.dart';
import 'package:sk_base_mobile/util/date.util.dart';
import 'package:sk_base_mobile/widgets/core/zt_number_input.dart';
import 'package:sk_base_mobile/widgets/core/zt_search_select.dart';
import 'package:sk_base_mobile/widgets/core/zk_date_picker.dart';
import 'package:sk_base_mobile/widgets/core/zt_text_input.dart';
import 'package:sk_base_mobile/widgets/gradient_button.dart';
import 'package:sk_base_mobile/screens/new_inventory_inout/new_inventory_inout_controller.dart';
import 'package:sk_base_mobile/util/screen_adaper_util.dart';

class NewInventoryInout extends StatelessWidget {
  final String inOrOut;
  NewInventoryInout({super.key, this.inOrOut = 'in'});

  final controller = Get.put(NewInventoryInoutController());

  @override
  Widget build(BuildContext context) {
    return SafeArea(
        top: false, // 设置为false以避免保留顶部状态栏的空间
        child: Scaffold(
            appBar: PreferredSize(
              preferredSize: Size.fromHeight(ScreenAdaper.height(
                  kToolbarHeight)), // 这里使用kToolbarHeight，它是不包含状态栏的AppBar的推荐高度
              child: AppBar(
                title: const Text('出入库登记'),
              ),
            ),
            resizeToAvoidBottomInset: true,
            body: SingleChildScrollView(
                child: Container(
              decoration: const BoxDecoration(
                color: Colors.white,
              ),
              child: Column(
                children: [
                  buildForm(),
                  Obx(() => GradientButton(
                        onPressed: () => {controller.create()},
                        isLoading: controller.loading.value,
                      ))
                ],
              ),
            ))));
  }

  Widget buildForm() {
    double formVerticalGap = 20.0;
    double formHorizontalGap = 20.0;
    final children = [
      Row(children: [
        Expanded(
          flex: 2,
          child: buildProjectPicker(),
        ),
        SizedBox(
          width: ScreenAdaper.width(formHorizontalGap),
        ),
        Expanded(
          flex: 3,
          child: buildProductPicker(),
        ),
      ]),
      SizedBox(
        height: ScreenAdaper.height(formVerticalGap),
      ),
      Row(
        children: [
          Expanded(flex: 1, child: buildQuantity()),
          SizedBox(
            width: ScreenAdaper.width(formHorizontalGap),
          ),
          Expanded(flex: 1, child: buildDatePicker()),
          SizedBox(
            width: ScreenAdaper.width(formHorizontalGap),
          ),
          Expanded(flex: 1, child: buildAgent()),
        ],
      ),
      SizedBox(
        height: ScreenAdaper.height(formVerticalGap),
      ),
      Row(
        children: [Expanded(child: buildRemark())],
      ),
      SizedBox(
        height: ScreenAdaper.height(formVerticalGap),
      ),
      buildImageUploadQueue()
    ];

    final child = Column(
      mainAxisAlignment: MainAxisAlignment.center,
      mainAxisSize: MainAxisSize.max,
      children: children,
    );

    return Form(
        key: controller.formKey,
        child: Container(
          padding: EdgeInsets.symmetric(
              vertical: ScreenAdaper.height(20),
              horizontal: ScreenAdaper.width(20)),
          alignment: Alignment.center,
          child: child,
        ));
  }

  /// 项目
  Widget buildProjectPicker() {
    return ZtSearchSelect(
        isRequired: true,
        textController: controller.projectTextController,
        labelText: '项目',
        itemBuilder: (itemData) => Container(
            padding: EdgeInsets.symmetric(
                vertical: ScreenAdaper.height(15),
                horizontal: ScreenAdaper.width(20)),
            child: Text(
              '${itemData?.name}',
              style: TextStyle(fontSize: ScreenAdaper.sp(20)),
            )),
        suggestionsCallback: (String keyword) {
          return controller.getProjects(keyword: keyword);
        },
        onClear: () {
          controller.payload.remove('projectId');
        },
        onSelected: (ProjectModel project) {
          controller.projectTextController.text = project.name ?? '';
          controller.payload['projectId'] = project.id;
        });
  }

  /// 产品
  Widget buildProductPicker() {
    return ZtSearchSelect(
        isRequired: true,
        textController: controller.productTextController,
        labelText: '产品',
        itemBuilder: (itemData) => ListTile(
              title: Text('${itemData?.name}'),
              subtitle: Text('${itemData?.company?.name ?? ''}'),
            ),
        suggestionsCallback: (String keyword) {
          return controller.getProducts(keyword: keyword);
        },
        onClear: () {
          controller.payload.remove('productId');
        },
        onSelected: (ProductModel product) {
          controller.productTextController.text = product.name ?? '';
          controller.payload['productId'] = product.id;
        });
  }

  /// 时间
  Widget buildDatePicker() {
    return ZtDatePicker(
      textController: controller.dateTextController,
      onClear: () {
        controller.payload.remove('time');
      },
      onDateSelected: (date) {
        if (date != null) {
          controller.dateTextController.text = DateUtil.format(date);
          controller.payload['time'] = controller.dateTextController.text;
        }
      },
    );
  }

  /// 数量
  Widget buildQuantity() {
    return ZtNumberInput(
      textController: controller.quantityTextController,
      labelText: '数量',
      isRequired: true,
    );
  }

  /// 经办人
  Widget buildAgent() {
    return ZtTextInput(
      textController: controller.agentTextController,
      labelText: '经办人',
      isRequired: true,
    );
  }

  /// 备注
  Widget buildRemark() {
    return ZtTextInput(
      textController: controller.remarkTextController,
      labelText: '备注',
      isTextArea: true,
    );
  }

  /// 照片上传
  Widget buildImageUploadQueue() {
    return Container(
      padding: EdgeInsets.symmetric(
          vertical: ScreenAdaper.height(20),
          horizontal: ScreenAdaper.width(20)),
      alignment: Alignment.center,
      child: Column(
        children: [
          Text(
            '*请拍照上传产品照片',
            style: TextStyle(
                fontSize: ScreenAdaper.sp(30),
                color: AppTheme.secondPrimaryColor),
          ),
          SizedBox(
            height: ScreenAdaper.height(10),
          ),
          Obx(() => SingleChildScrollView(
                controller: controller.uploadScrollController,
                scrollDirection: Axis.horizontal,
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.center,
                  children: [
                    ...controller.uploadImgFilesPath
                        .map((String path) => builderImagePreview(path))
                        .toList(),
                    buildImageUploader()
                  ],
                ),
              )),
        ],
      ),
    );
  }

  /// 照片预览队列Item
  Widget builderImagePreview(String path) {
    return Stack(
      children: [
        Container(
          margin: EdgeInsets.symmetric(horizontal: ScreenAdaper.width(5)),
          width: ScreenAdaper.width(200),
          height: ScreenAdaper.width(200),
          decoration: BoxDecoration(
            borderRadius: BorderRadius.circular(20),
            border: Border.all(width: 1.0, color: AppTheme.dividerColor),
            // color: AppTheme.primaryColor,
          ),
          child: Center(
            child: Container(
              decoration: BoxDecoration(
                borderRadius: BorderRadius.circular(10),
                image: DecorationImage(
                  fit: BoxFit.cover,
                  image: FileImage(File(path)),
                ),
              ),
            ),
          ),
        ),
        Positioned(
            top: ScreenAdaper.height(5),
            right: ScreenAdaper.width(5),
            child: GestureDetector(
              onTap: () {
                controller.uploadImgFilesPath.remove(path);
              },
              child: Icon(
                Icons.close,
                shadows: [
                  Shadow(
                    color: Colors.black,
                    offset: Offset(1, 1),
                    blurRadius: 3,
                  ),
                  Shadow(
                    color: Colors.black,
                    offset: Offset(-1, -1),
                    blurRadius: 3,
                  ),
                ],
                size: ScreenAdaper.sp(40),
                color: AppTheme.nearlyWhite,
              ),
            ))
      ],
    );
  }

  // 上传照片控制器
  Widget buildImageUploader() {
    return GestureDetector(
        onTap: () {
          controller.photoPicker();
        },
        child: Container(
            margin: EdgeInsets.symmetric(horizontal: ScreenAdaper.width(5)),
            width: ScreenAdaper.width(200),
            height: ScreenAdaper.width(200),
            decoration: BoxDecoration(
              borderRadius: BorderRadius.circular(20),
              border: Border.all(width: 1.0, color: AppTheme.dividerColor),
              // color: AppTheme.primaryColor,
            ),
            child: Center(
                child: Icon(
              Icons.add_a_photo_rounded,
              size: ScreenAdaper.sp(60),
              color: AppTheme.primaryColor,
            ))));
  }
}
